---------------------------------------------------------
---------------------------------------------------------: Command not found
-- File: banker.vhd						               --
-- Last modified: 22 February 2005				       --
-- By: Steven C. Bunch, Lanie Britton, and Chris Beall --
---------------------------------------------------------
LIBRARY IEEE;
USE  IEEE.STD_LOGIC_1164.all;
USE  IEEE.STD_LOGIC_ARITH.all;
USE  IEEE.STD_LOGIC_UNSIGNED.all;
USE  work.controlFuncs.ALL;

ENTITY banker IS
    Generic (n          :integer :=3); 				                --Number of players including house
        
    PORT (  Clk         : IN STD_LOGIC; 				            --Global clock input
        
        Reset           : IN STD_LOGIC; 				            --Global reset
        
        playerId        : IN STD_LOGIC_VECTOR (n-1 downto 0);       --from controller
        
        Action          : IN STD_LOGIC_VECTOR (2*n-1 downto 0);     --from score keeper
        
        TakeAction      : IN STD_LOGIC; 				            --from score keeper
        
        cgNewHand       : IN STD_LOGIC; 			 	            --from controller    
        
        bet             : IN STD_LOGIC_VECTOR (11 DOWNTO 0);	    --first two bits are 0
                                                                    --last ten bits are from players
        
        BetReady	    : IN STD_LOGIC;				                --From players, '1' for bet ready '0' for not ready
        
        balance         : OUT STD_LOGIC_VECTOR (11 downto 0); 	    --first two bits are 0
                                                                    --last ten bits go to players
                                                                    --Max is 1024
        
        Ties	        : OUT STD_LOGIC;				            --To players, '1' for ties reported '0' for not
        
        HiPlayerBal     : OUT STD_LOGIC_VECTOR (11 DOWNTO 0);	    --Balance amount of highest player balance
        
        GameOver        : OUT STD_LOGIC; 			     	        --The whole game is over
                                                                    --goes to controller
        
        balancerdy      : OUT STD_LOGIC                             --goes to controller and players
	);		
        
        
END banker;

ARCHITECTURE a OF banker IS
    type matrix is array (integer range n-1 downto 0) of STD_LOGIC_VECTOR (11 downto 0);
    signal  initialBank    : STD_LOGIC_VECTOR (11 downto 0);	-- Indicates starting value of each player's pot
    signal  CheckPlayer    : STD_LOGIC_VECTOR (n-2 downto 0);	-- NOT USED
    signal  End_Game       : STD_LOGIC;					        -- End of hand
    signal  HP_Stop        : STD_LOGIC;					        -- NOT USED
    signal  bet_store      : matrix;					        -- NOT USED
    signal  Togbalrdy      : STD_LOGIC;					        -- Toggles whether the balance is ready to be checked
    signal  Count          : STD_LOGIC_VECTOR (6 DOWNTO 0);	    -- Counts number of games played
    signal  End_BJ         : STD_LOGIC;					        -- Ends all playing of the Blackjack game
    signal  PL_Out         : STD_LOGIC_VECTOR (n-1 downto 0);	-- Indicates whether or not a player has any money left
    signal  bank           : matrix;					        -- Matrix that stores each player's total present chip count
    signal  betstore       : matrix;					        -- Stores each player's present bet
    signal  BetFlag        : STD_LOGIC;					        -- Indicates if a player wants to bet or not
    signal  EndHand        : STD_LOGIC;					        -- Ends a given hand when it has been won by someone
    signal  Chkbalrdy      : STD_LOGIC;					        -- Indicates whether the balance should be checked
    
    begin
        ---------------------------
        -- Handling Total Scores --
        ---------------------------
        
	
	process (Clk, Reset, bet, TakeAction, cgNewHand,PlayerId)
        variable ActionDone: STD_LOGIC;
        variable hi_score: STD_LOGIC_VECTOR(11 DOWNTO 0) := "000000000000";  
        variable tie_holder: STD_LOGIC := '0';
	variable BalChkId:  INTEGER := 0;		-- Balance check Id: from 0 to n-1;
	variable HiBalFlag: STD_LOGIC := '0';		-- To indicate whether to check the highest balance of all the players;


        begin

            if (Clk = '1' AND Clk'EVENT) then
                --------------------------------------------------------------------
                -- If reset is toggled then reset each players bank amount to 127 --
                --------------------------------------------------------------------
                if (Reset = '1') then
                    initialBank <= "000001111111"; --127
                    for i in n-1 downto 0 loop
                        bank(i) <= initialBank; 	--stores 127 for each player in the bank
                    end loop;
							PL_OUT <= (OTHERS => '0');  	-- added by liu 2/3
							End_Game <= '0';                    -- Game is starting (therfore not over)
               	   HiPlayerBal <= "000000000000";      -- Added 2/25/05
							HiBalFlag := '1';			-- added by liu 2/4
							Ties <= '0';                        -- Added 2/25/05

                else
                    -----------------------------------------------------------------------------
                    -- If reset is not toggled then restore bank value to the player's balance --
                    -- and do some handshaking with the controller and players			       --
                    ----------------------------------------------------------------------------- 
                    
                    -- what if PlayerID is not '1', should we DQ them, since if you leave 127 in their account,
                    -- they could actually end up winning the game!?!?!?!? 
                    
                    -- CORRECTED
                    
                    for i in n-1 downto 0 loop
                        if PlayerId(i) = '1' then --checking if a player is present
                            balance <= bank(i); --storing bank value to player balance
                            if (Togbalrdy = '1') then
                                if (Chkbalrdy = '0') then
                                    balancerdy <= '1';
                                    Chkbalrdy  <= '1';
                                end if;
                            end if;
                       -- else                            -- added JLB 2/25   -- delete by liu 2/2
                       --     bank(i) <= "000000000000";  -- added JLB 2/25   -- delete by liu 2/2
                       --     balance <= bank(i);         -- added JLB 2/25   -- delete by liu 2/2
                        end if;
                    end loop;
                    if (Chkbalrdy = '1') then
                        balancerdy <= '0';
                        Chkbalrdy  <= '0';
                    end if;
                    if (cgNewHand = '1') then 
                        BetFlag <= '1';
                    end if;
		    
                end if;
                -----------------------------------------------------------------------------------
                -- If it's a new hand then store the bet from each player into a betstore vector --
                -- and restore bank value to the player's balance and do some handshaking with   --
                -- the controller and players									                 --
                -----------------------------------------------------------------------------------
                if BetFlag ='1' then
                    for i in n-1 downto 0 loop
                        if PlayerId(i) = '1' then
                            if BetReady = '1' then
                                betstore(i) <= bet;
                                balance <= bank(i);
                                if (Togbalrdy = '1') then
                                    if (Chkbalrdy = '0') then
                                        balancerdy <= '1';
                                        Chkbalrdy <= '1';
                                    end if;
                                end if;
--                              if i = n-1 then			-- moved down: when bet finished, cannot check balance;
--                                    balancerdy <= '0';
--                                    Chkbalrdy <= '0';
--                                   Togbalrdy <= '0';
--                              end if;
                            end if;
                        end if;
                    end loop;
                    if PlayerId(n-1) = '1' then
                        BetFlag <= '0';
								balancerdy <= '0';
                        Chkbalrdy <= '0';
                        Togbalrdy <= '0';   
                    end if;
                end if;
                -------------------------------
                -- not sure why this is here --
                -------------------------------
                if EndHand = '1' then
                    	balancerdy <= '0';
                    EndHand <= '0';
                end if;        
                --------------------------------------------------------------------------------
                -- Once the scorekeeper has determined the winner and payout add or substract --
                -- bet amounts from each player's bank account balance and store the balance  --
                -- back into the bank matrix									--
                --------------------------------------------------------------------------------
                if (TakeAction = '1') then  
                    if (ActionDone = '0') then                       
                        for i in n-2 downto 0 loop
                            --case when player = Win
                            if Action(2*i + 1 downto 2*i) = "11" then  -- blackjack
                                if (bank(i) >= betstore(i)) then    
                                    bank(i) <= ( betstore(i) + betstore(i)) + bank(i);
                                else
                                    bank(i) <= ( bank(i) + bank(i)) + bank(i);
                                end if;		
                                --case when player wins (no BJ)
                            elsif 
                                Action(2*i + 1 downto 2*i) = "10"    then -- win
                                if (bank(i) >= betstore(i)) then    
                                    bank(i) <= ( betstore(i)) + bank(i);
                                else
                                    bank(i) <=  bank(i) + bank(i);
                                    
                                    --case when player = push (Tie)
                                end if;
                            elsif 
                                Action(2*i + 1 downto 2*i) = "01" then -- tie
                                bank(i) <= bank(i);			
                                
                                --case when player = Lose
                            elsif 
                                Action(2*i + 1 downto 2*i) = "00" then -- lose
                                if (bank(i) >= betstore(i)) then                     
                                    bank(i) <= bank(i) - betstore(i);
                                else
                                    bank(i) <= "000000000000";
                                    PL_Out(i) <= '1';
                                end if;
                            end if;
                        end loop;
--                        balancerdy <= '1';		-- removed to the IF below;
--                        Chkbalrdy  <= '1';		-- removed to the IF below;
--                        TogBalrdy  <= '1';		-- removed to the IF below
                        EndHand    <= '1';
                        ActionDone := '1';
                    end if;
                else
                    ActionDone := '0';
                end if;
					------------------------------------------------------------------------------
					-- If a hand is finished, calculate the higheset balance of all the players --
					-- And set Balancerdy to '1'
					------------------------------------------------------------------------------
					 
					if (EndHand = '1') then               -- If Hand has finished...
						End_Game <= '1';	          -- Game is over
						HiBalFlag := '1';		  -- Need to check the highest balance now; Added by liu 2/4
						hi_score := "000000000000";
					else
						End_Game <= '0';                  -- If Hand has not finished, the game is not over
					end if;
					
					if HiBalFlag = '1' then
						if bank(BalChkId) > hi_score then
							hi_score := bank(BalChkId);
						end if;
						
						BalChkId := BalChkId + 1;
						
						if BalChkId = n-1 then
							HiBalFlag := '0';
							BalChkId := 0;
							HiPlayerBal <= hi_score;
							TogBalrdy  <= '1';		-- moved from the IF above;
							balancerdy <= '1';		-- moved from the IF above;
							Chkbalrdy <= '1';		-- moved from the IF above
						end if;
					end if;
		    				
            end if;
        end process;
        
        ------------------------------------------------------------
        -- Set End_Game to high after each hand is finished       --
        -- Will be used to determine if GameOver should be set    --
        ------------------------------------------------------------
--        process(Clk, Reset)
--        begin
--	    
--	    if (Clk'EVENT and Clk = '1') then
--	    	if (Reset = '1') then                       	-- If Game is being reset...
--           	else
--            	end if; 
--	    end if;
--        end process;
--	for i in n-1 downto 0 loop		-- changed 2/4 by liu;
--          if PlayerId(i) = '1' then
--                if bank(i) = hi_score then 	-- Added 2/25/05
--                      tie_holder := '1';      -- Added 2/25/05
--                      hi_score := bank(i);    -- Added 2/25/05                            
--                elsif bank(i) >= hi_score then
--                      hi_score := bank(i);
--                      tie_holder := '0';      -- Added 2/25/05
--               end if;			-- The hiplayerbal always equals the balance of player n-1 because endhand last for 1 cyc.
--          end if;
--      end loop;
--        
--     HiPlayerBal <= hi_score;                -- Changed 2/25/05 - Moved from 'end loop' above ? Correct ?
--    Ties <= tie_holder;                      -- Added 2/25/05

        
        ------------------------------------------------------------
        -- OUTPUT GameOver for the Controller based on a flag set --
        -- in a process under "banker.vhd" (End_BJ)               --
        ------------------------------------------------------------
        process(Clk, Reset, End_BJ)
        begin
            if (Reset = '1') then                       -- If Game is being reset...
                GameOver <= '0';                        -- Game is starting (therfore not over)
            else
                if (End_BJ = '1') then                  -- If End_BJ signal (which is set by process below) is high,
                    GameOver <= '1';                    -- the game is over
                else
                    GameOver <= '0';                    -- The game is not over if End_BJ signal is not high
                end if;
            end if;
        end process;
        
        -------------------------------------------------------------------------
        -- this process counts the number of games that have been completed
        -------------------------------------------------------------------------
        process (Clk, Reset, End_Game)
        begin
            if (Reset = '1') then       -- if reset was pushed, reset the counter
                Count <= "0000000";
            elsif (Clk'EVENT and Clk = '1') then
                if (End_Game = '1') then       -- if the end of a game is detected..
                    Count <= Count + "0000001";  -- increment the Count counter
                else
                    Count <= Count;              -- otherwise, hold Count
                end if;
            end if;
        end process;
        
        ---------------------------------------------------------------------------
        -- this process ends everything after 100 games or after all players use --
        -- up their chips, or when a player has 1024 chips				 --
        ---------------------------------------------------------------------------
        process (Clk, Reset, PL_Out, Count)
        variable MaxB :STD_LOGIC;
        variable Plcheck: INTEGER;
        begin
	    PLcheck := 0;          -- added by liu 2/3
            if (Reset = '1') then  -- reset the game
                End_BJ <= '0';
            elsif (Clk'EVENT and Clk = '1') then
                bankmax : for i in n-2 downto 0 loop  -- test for more than 1023 chips for each player
                if bank(i) >= "001111111111" then
                    MaxB := '1';      -- if a player has over 1023 chips set MaxB to 1 
                    exit bankmax;     -- and exit the bankmax loop
                end if;               -- otherwise keep looping
            end loop bankmax;
            
            --	for i in n-2 downto 0 loop
            for i in n-2 downto 0 loop
                if PL_Out(i) = '1' then    -- if the current player is out, increment the Plcheck counter
                    Plcheck := PLcheck + 1;
--	    	    ASSERT (PLcheck <= 1)
--	    	    	REPORT "PLcheck sucks"		    
--	    	    	SEVERITY note;                       -- added by liu 2/3 for debug
                end if;
            end loop;
	    
	    		
            if (Count = "1100100") OR (Plcheck = n-1) OR (MaxB='1') then   -- if the count is 100, end game
                MaxB := '0';        -- if we've checked all players for not having any chips left, end game
                End_BJ <= '1';  -- if some has the max number of chips, end game, reset flag for max # of chips
                Plcheck := 0;       -- reset the Plcheck counter
            else
                End_BJ <= '0';      -- otherwise, game not over!
            end if;
            --	end loop;
        end if;
    end process;
    
    -------------------------------------------------------------------------------
    -- this process displays the balance for each player at the end of each game --
    -------------------------------------------------------------------------------
    process (Clk, End_BJ)
    variable currBal: integer := 0;     -- the bank(i) value will be stored here to display
    variable its: std_logic := '0';
    begin 
        if (End_BJ = '1' AND its = '0') then
            for i in 0 to n-1 loop          -- loop through each player and get the balance
                currBal := CONV_INTEGER(bank(i));   -- bank is the matrix that holds the balance for each player i
                report "Player " & integer'image(i) & " = " & integer'image(currBal) 
                & " chips";
            end loop;
            its := '1';
        end if;
    end process;
    
end a;

