----------------------------------------------------------------------------------
-- ScoreKeeper Module for BlackJack 
--
-- ECE 552 - Spring 2006 and 2007 Semesters
--
-- Scorekeeper module alerts the controller immediately of a player bust, after the
-- hand is completed it alerts the banker so the banker can begin calculating, and 
-- provides the banker with the results of the hand (blackjack, win, loss or tie)
----------------------------------------------------------------------------------


LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
USE std.textio.all;

ENTITY Bjsk IS

    GENERIC(
        n : INTEGER := 6  -- Number of players
    );

    PORT(
        Clk        : IN  STD_LOGIC;                        -- Global Clk
        Reset      : IN  STD_LOGIC;                        -- Global Reset
        cgIssue    : IN  STD_LOGIC;                        -- Card has been issued (from Controller)
        PlStand    : IN  STD_LOGIC;                        -- To inform that all players are standing (from Controller)
        GameOver   : IN  STD_LOGIC;                        -- Whole game is over(from Banker)
        Card       : IN  STD_LOGIC_VECTOR (3 DOWNTO 0);    -- Card Value (from Card Generator)
        PlayerId   : IN  STD_LOGIC_VECTOR (n-1 DOWNTO 0);  -- (from Controller)
	
	
        TakeAction : OUT STD_LOGIC;                        -- To the banker to start calculating
        skBust     : OUT STD_LOGIC;                        -- To the controller to infrm about a player bust
        Action     : OUT STD_LOGIC_VECTOR (2*n-1 DOWNTO 0) -- Output to the Banker

);

END Bjsk;

ARCHITECTURE behavior OF Bjsk IS

    TYPE matrix  IS ARRAY (integer RANGE n-1 DOWNTO 0) OF STD_LOGIC_VECTOR(4 DOWNTO 0);
    TYPE matrix1 IS ARRAY (integer RANGE n-1 DOWNTO 0) OF STD_LOGIC_VECTOR(2 DOWNTO 0);
--    TYPE matrix2 IS ARRAY (integer RANGE n-1 DOWNTO 0) OF STD_LOGIC_VECTOR(2 DOWNTO 0);

    SIGNAL Optotal      : matrix;    -- To keep track of total card values
    SIGNAL BJflagmatrix : matrix1;   -- To check number of cards for Blackjack
    SIGNAL cardIssued   : STD_LOGIC; -- Indicates a card has been passed to a player (or the HP)
    SIGNAL InitOptotal  : STD_LOGIC; -- Indicates hand has finished (TakeAction)



BEGIN

-- Card Issued
-- if card is issued then check playerId and calculate the total card value
    PROCESS (Reset, GameOver, PlayerId, cardIssued, Clk, Card, cgIssue, PlStand)
        VARIABLE Acematrix    : matrix1; -- Stores information about who received an Ace for later calculations (1 or 11)
        VARIABLE IssueChecked : STD_LOGIC;  -- Makes sure same card is not counted twice
        VARIABLE CardValue    : STD_LOGIC_VECTOR (4 DOWNTO 0); -- Stores value of card (Card)
        VARIABLE TempOptotal  : STD_LOGIC_VECTOR (4 DOWNTO 0); -- Stores values for Optotal
        VARIABLE PlayerDiff   : STD_LOGIC_VECTOR (4 DOWNTO 0); -- If no one gets BJ or 21, calculates difference from 21
        VARIABLE HPDiff       : STD_LOGIC_VECTOR (4 DOWNTO 0); -- same as above for HP
		VARIABLE HP_BJ	      : STD_LOGIC;  -- Flag to indicate if House Player has a blackjack
	--Following lines added for output
	alias swrite is write [line, string, side, width];
	variable s : line;
	file fout : text is out "bjskOut.txt";

    BEGIN
    
        IF ((Reset = '1') OR (GameOver = '1')) THEN  --- Reset all signals and start over (per hand)
            FOR i IN n-1 DOWNTO 0 LOOP
                Optotal(i)      <= "00000";
                BJflagmatrix(i) <= "000";
                AceMatrix(i)    := "000";
		          HP_BJ           := '0';
            END LOOP;

        ELSE
            IF (Clk = '1' AND Clk'EVENT) THEN   --- Reset all signals and start over (per card)
                IF (InitOptotal = '1') THEN
                    Optotal      <= (OTHERS => "00000");
                    BJflagmatrix <= (OTHERS => "000");
                    Acematrix    := (OTHERS => "000");
                    InitOptotal  <= '0';
                    TakeAction   <= '0';
		              HP_BJ        := '0';
                END IF;
		
                IF (cgIssue = '1' ) THEN   -- If card is issued
                    IF (IssueChecked = '0') THEN -- So that Operations performed only once after cgIssue goes high irrespective of the number of clock cycles 
                        FOR i IN n-1 DOWNTO 0 LOOP  --To check which player has the card issued
                           	IF PlayerId(i) = '1' THEN  
                                    CardValue             := "00000";

                                -- insures that if card recieved is 10 or higher, it only counts as 10
                                    IF (Card > "1001") THEN
                                        CardValue(3 DOWNTO 0) := "1010";
                                    ELSE
                                        CardValue(3 DOWNTO 0) := Card;
                                    END IF;
                                
                                    IF (CardValue = "00001") THEN  -- To check if the card is an Ace which is a special condition
                                       	TempOptotal := Optotal(i) + "01011";
                                   	   IF (BJflagmatrix(i) < "011") THEN
                                           BJflagmatrix(i) <= BJflagmatrix(i) + "001"; -- increment card count
                                   	   END IF;

                                   	   IF TempOptotal > "10101" THEN  --If total card value is greater than 21
                                          TempOptotal := TempOptotal - "01011" + "00001"; --Change the card value of the Ace to 1 instead of 11
                                   	   ELSE
                                          Acematrix(i) := Acematrix(i) + "001"; -- If value of 11 is taken keep track for later use
                                   	   END IF;
                                   	    Optotal(i) <= TempOptotal; 
                                    ELSE
                                   	   IF (BJflagmatrix(i) < "011") THEN
                                           BJflagmatrix(i) <= BJflagmatrix(i) + "001"; -- increment card count
                                   	   END IF;
                                	       TempOptotal := Optotal(i) + CardValue;
                                   	    Optotal(i)  <= Optotal(i) +  CardValue;
                                   	   IF (TempOptotal > "10101") THEN  -- to check if the total card value is greater than 21
                                            Aces : FOR j IN 7 DOWNTO 0 LOOP
                                           	   IF (Acematrix(i) > "000")  THEN-- To check if the player has any Aces whose value can be subtracted
                                                  	TempOptotal  := TempOptotal - "01010";
                                                   Acematrix(i) := Acematrix(i) - "001";	
                                                       IF TempOptotal <= "10101" THEN
                                                         	EXIT Aces;
                                                       END IF; 
                                              	ELSE
                                                    EXIT Aces;
                                   	           END IF;
                                           END LOOP Aces;
                                   	   END IF; 

                                   	    Optotal(i) <= TempOptotal ;
                                    END IF;

                           	END IF; -- END PlayerID IF
                        END LOOP; -- END LOOP for determining which player has been issued a card
                        IssueChecked := '1';
                    END IF; --END of issue checked

                ELSE
                    IssueChecked := '0';
                    cardIssued   <= '0';

                END IF;--END of card issued statement


--when all players stand and controller waits for balance ready to be set high by the banker

                IF (PlStand = '1' ) THEN-- to check if the hand is over and scores can be calculated
		    swrite(s, "HAND OVER - totals being calculated with House Player first");
		    writeline(fout, s);
                    IF (Optotal(n - 1) = "10101")THEN  --to check if dealer has a blackjack
                        IF (BJflagmatrix(n - 1) = "010") THEN
				HP_BJ := '1';
				swrite(s, "House player has a Blackjack");
				writeline(fout, s);
                        END IF;
                    END IF;
                    FOR i IN n-1 DOWNTO 0 LOOP
			swrite(s, "Player ");
			write(s, i);
			swrite(s, " total is ");
			write(s, CONV_INTEGER(Optotal(i)));
			writeline(fout, s);
			swrite(s, "Player ");
			write(s, i);
                        IF (Optotal(i) = "10101") THEN  
                           	IF (BJflagmatrix(i) = "010") THEN  -- if the player has a blackjack
                                    IF (HP_BJ = '1') THEN -- if house player has blackjack
                                       	Action(2*i+1 DOWNTO 2*i) <= "01"; -- tie between houseplayer and player
					swrite(s, " ties");
                                    ELSE
                                   	Action(2*i+1 DOWNTO 2*i) <= "11"; --player has blackjack
					swrite(s," wins with a blackjack");
                                    END IF;
                           	ELSE
                                    IF (HP_BJ = '1' ) THEN
                                   	Action(2*i+1 DOWNTO 2*i) <= "00"; -- player looses
					swrite(s, " loses");
                                    ELSE
                                   	Action(2*i+1 DOWNTO 2*i) <= "10"; -- player wins
					swrite(s, " wins");
                                    END IF;
                           	END IF;
                        ELSE
                           	IF (HP_BJ = '1') THEN
                                Action(2*i+1 DOWNTO 2*i) <= "00"; -- player looses
				swrite(s, "loses");
                           	ELSE
                                IF (Optotal(i) > "10101") THEN  -- if player has card value of greater than 21
                                   	IF (Optotal(n - 1) > "10101") THEN -- if house player is also over 21
                                            Action(2*i+1 DOWNTO 2*i) <= "01"; -- player ties
					    swrite(s, " ties");
                                   	ELSE
                                            Action(2*i+1 DOWNTO 2*i) <= "00"; -- player loses
					    swrite(s, " loses");
                                   	END IF;
                                ELSE                      -- Take difference of cards to check who is closer to 21
                                   	PlayerDiff := "10101" - Optotal(i);
                                   	HPDiff     := "10101" - Optotal(n - 1);
                                   	IF (HPDiff > PlayerDiff) THEN
                                            Action(2*i+1 DOWNTO 2*i) <= "10"; --player wins
					    swrite(s, " wins");
                                   	ELSE
                                            Action(2*i+1 DOWNTO 2*i) <= "00"; --player looses
					    swrite(s, " loses");
                                   	END IF;
                                END IF;
                           	END IF;
                        END IF;
			writeline(fout, s);
                    END LOOP;
                    InitOptotal <= '1';
                    TakeAction  <= '1';   -- SEND signal to banker
                END IF; -- PlStand
		
		
		
            END IF;     -- clock
        END IF;         -- reset/gameover
    END PROCESS;

--PROCESS to set or reset skBust
    PROCESS(Clk, playerId)
        VARIABLE bustFlag : STD_LOGIC := '0';
    BEGIN
        IF(Clk = '1' AND Clk'EVENT) THEN
            FOR i IN n-1 DOWNTO 0 LOOP  --To check which player has the card issued
                IF PlayerId(i) = '1' THEN  
                    IF ((Optotal(i) > "10101") AND bustFlag = '0') THEN 
-- If total greater than 21 SEND signal to the controller that player is bust
                       	bustFlag := '1';
                       	skBust   <= '1';
                    ELSE
                       	bustFlag := '0';
                       	skBust   <= '0';
                    END IF;	
                END IF;
            END LOOP;
        END IF;
    END PROCESS;	
END behavior;
