LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_arith.ALL;
USE ieee.std_logic_unsigned.ALL;
use work.controlFuncs.all;


entity top_TB is
    
end top_TB;

architecture structure of top_TB is
  CONSTANT nPlayers : INTEGER := 3;     -- Number of Players
  COMPONENT controller
    GENERIC (
      nPlayers : INTEGER);
    PORT (
      clock       : IN  STD_LOGIC;
      enable      : IN  STD_LOGIC;
      reset       : IN  STD_LOGIC;
      bistEnable  : IN  STD_LOGIC;
      pHit        : IN  STD_LOGIC_VECTOR(nPlayers-1 DOWNTO 0);
      pStand      : IN  STD_LOGIC_VECTOR(nPlayers-1 DOWNTO 0);
      issued      : IN  STD_LOGIC;
      gameOver    : IN  STD_LOGIC;
      balanceRdy  : IN  STD_LOGIC;
      skBust      : IN  STD_LOGIC;
      playerID    : OUT STD_LOGIC_VECTOR(nPlayers-1 DOWNTO 0);
      globalReset : OUT STD_LOGIC;
      cgIssue     : OUT STD_LOGIC;
      cgNewHand   : OUT STD_LOGIC;
      cgBist      : OUT STD_LOGIC;
      handOver    : OUT STD_LOGIC;
	  shuffle	  : OUT STD_LOGIC
	  );
  END COMPONENT;

  component Bjsk
    generic (
      n : integer);
    port (
      Clk        : in  std_logic;
      Reset      : in  std_logic;
      Card       : in  std_logic_vector (3 downto 0);
      PlayerId   : in  std_logic_vector (n-1 downto 0);
      cgIssue    : in  std_logic;
      PlStand    : in  std_logic;
      GameOver   : in  std_logic;
      Action     : out std_logic_vector (2*n-1 downto 0);
      TakeAction : out std_logic;
      skBust     : out std_logic);
  end component;
  
  component banker
    generic (
      n : integer);
    port (
      Clk        : IN  STD_LOGIC;
      Reset      : IN  STD_LOGIC;
      playerId   : IN  STD_LOGIC_VECTOR (n-1 downto 0);
      Action     : IN  STD_LOGIC_VECTOR (2*n-1 downto 0);
      TakeAction : IN  STD_LOGIC;
      cgNewHand  : IN  STD_LOGIC;
      bet        : IN  STD_LOGIC_VECTOR (11 DOWNTO 0);
	  betReady   : IN STD_LOGIC;
      balance    : OUT STD_LOGIC_VECTOR (11 downto 0);
	  --Ties       : OUT STD_LOGIC;
	  HiPlayerBal: OUT STD_LOGIC_VECTOR(11 downto 0);
      GameOver   : OUT STD_LOGIC;
      balancerdy : OUT STD_LOGIC);
  end component;

  component card_generator is
  port(	in_clock : in std_logic;    
	in_reset : in std_logic; 				-- reset to new game
	in_issue : in std_logic; 				-- request a new card
	in_bist : in std_logic;			-- test mode (standard cards)
	main_reset : in std_logic;


	shuffle : in std_logic;
	in_num_twos : in std_logic_vector(4 downto 0);
	in_num_threes : in std_logic_vector(4 downto 0);
	in_num_fours : in std_logic_vector(4 downto 0);
	in_num_fives : in std_logic_vector(4 downto 0);
	in_num_sixes : in std_logic_vector(4 downto 0);
	in_num_sevens : in std_logic_vector(4 downto 0);
	in_num_eights : in std_logic_vector(4 downto 0);
	in_num_nines : in std_logic_vector(4 downto 0);
	in_num_tens : in std_logic_vector(4 downto 0);
	in_num_jacks : in std_logic_vector(4 downto 0);
	in_num_kings : in std_logic_vector(4 downto 0);
	in_num_queens : in std_logic_vector(4 downto 0);
	in_num_aces : in std_logic_vector(4 downto 0);

	out_card 	: out std_logic_vector(3 downto 0); -- card output on issue
	out_house : out std_logic_vector(3 downto 0); -- dealer card on reset
	out_issued : out std_logic; -- signals out_card is ready (pulse high)
	over_reset : out std_logic;
	ccard:out std_logic_vector(3 downto 0);
	dealttotal_out : in std_logic_vector(7 downto 0);
	house_card_dealt : out std_logic
	);
end component;

component cardcounter IS
   
   PORT(clock 		: IN	STD_LOGIC;
   		reset  		: IN    STD_LOGIC;		
   		issued		: IN	STD_LOGIC;
   		card		: IN	STD_LOGIC_VECTOR (3 downto 0);
   		over_reset	: in std_logic;
   		house_card_dealt : in std_logic;
   		stat_ace 	: OUT 	STD_LOGIC_VECTOR (4 downto 0);
		stat_two 	: OUT 	STD_LOGIC_VECTOR (4 downto 0);
		stat_three 	: OUT 	STD_LOGIC_VECTOR (4 downto 0);
		stat_four 	: OUT 	STD_LOGIC_VECTOR (4 downto 0);
		stat_five 	: OUT 	STD_LOGIC_VECTOR (4 downto 0);
		stat_six 	: OUT 	STD_LOGIC_VECTOR (4 downto 0);
		stat_seven 	: OUT 	STD_LOGIC_VECTOR (4 downto 0);
		stat_eight 	: OUT 	STD_LOGIC_VECTOR (4 downto 0);
		stat_nine 	: OUT 	STD_LOGIC_VECTOR (4 downto 0);
		stat_ten 	: OUT 	STD_LOGIC_VECTOR (4 downto 0);
		stat_jack	: OUT 	STD_LOGIC_VECTOR (4 downto 0);
		stat_queen	: OUT 	STD_LOGIC_VECTOR (4 downto 0);
		stat_king 	: OUT 	STD_LOGIC_VECTOR (4 downto 0);
   	  	face_chance : out std_logic_vector(2 downto 0);
		dealttotal_out : out std_logic_vector(7 downto 0)
   );  
   
END component;


  COMPONENT housePlayer
    PORT (
      clock                         : IN  STD_LOGIC;
      reset                         : IN  STD_LOGIC;
      turn                          : IN  STD_LOGIC;
      issued                        : IN  STD_LOGIC;
      card                          : IN  STD_LOGIC_VECTOR(3 DOWNTO 0);
      dealerShow                    : IN  STD_LOGIC_VECTOR(3 DOWNTO 0);
      NumA, Num2, Num3, Num4, Num5  : IN  STD_LOGIC_VECTOR(4 DOWNTO 0);
      Num6, Num7, Num8, Num9, Num10 : IN  STD_LOGIC_VECTOR(4 DOWNTO 0);
      NumJ, NumQ, NumK              : IN  STD_LOGIC_VECTOR(4 DOWNTO 0);
      numHands                      : IN  STD_LOGIC_VECTOR(6 DOWNTO 0);
      balance                       : IN  STD_LOGIC_VECTOR(11 DOWNTO 0);
      balanceRdy                    : in  STD_LOGIC;
      hit                           : OUT STD_LOGIC;
      stand                         : OUT STD_LOGIC;
      bet                           : OUT STD_LOGIC_VECTOR(11 DOWNTO 0));
  END COMPONENT;

  COMPONENT basicPlayer
    PORT (
      clock                         : IN  STD_LOGIC;
      reset                         : IN  STD_LOGIC;
      turn                          : IN  STD_LOGIC;
      issued                        : IN  STD_LOGIC;
      card                          : IN  STD_LOGIC_VECTOR(3 DOWNTO 0);
      dealerShow                    : IN  STD_LOGIC_VECTOR(3 DOWNTO 0);
      NumA, Num2, Num3, Num4, Num5  : IN  STD_LOGIC_VECTOR(4 DOWNTO 0);
      Num6, Num7, Num8, Num9, Num10 : IN  STD_LOGIC_VECTOR(4 DOWNTO 0);
      NumJ, NumQ, NumK              : IN  STD_LOGIC_VECTOR(4 DOWNTO 0);
      numHands                      : IN  STD_LOGIC_VECTOR(6 DOWNTO 0);
      balance                       : IN  STD_LOGIC_VECTOR(11 DOWNTO 0);
      balanceRdy                    : in  STD_LOGIC;
      hit                           : OUT STD_LOGIC;
      stand                         : OUT STD_LOGIC;
	  --Ties							: IN STD_LOGIC;
	  HiPlayerBal					: IN STD_LOGIC_VECTOR(11 downto 0);
	  betReady                      : OUT STD_LOGIC;
	  face_chance					: IN STD_LOGIC_VECTOR(2 downto 0);
      bet                           : OUT STD_LOGIC_VECTOR(11 DOWNTO 0));
  END COMPONENT;

  COMPONENT fileBasicPlayer
    PORT (
      clock                         : IN  STD_LOGIC;
      reset                         : IN  STD_LOGIC;
      turn                          : IN  STD_LOGIC;
      issued                        : IN  STD_LOGIC;
      card                          : IN  STD_LOGIC_VECTOR(3 DOWNTO 0);
      dealerShow                    : IN  STD_LOGIC_VECTOR(3 DOWNTO 0);
      NumA, Num2, Num3, Num4, Num5  : IN  STD_LOGIC_VECTOR(4 DOWNTO 0);
      Num6, Num7, Num8, Num9, Num10 : IN  STD_LOGIC_VECTOR(4 DOWNTO 0);
      NumJ, NumQ, NumK              : IN  STD_LOGIC_VECTOR(4 DOWNTO 0);
      numHands                      : IN  STD_LOGIC_VECTOR(6 DOWNTO 0);
      balance                       : IN  STD_LOGIC_VECTOR(11 DOWNTO 0);
      balanceRdy                    : in  STD_LOGIC;
      hit                           : OUT STD_LOGIC;
      stand                         : OUT STD_LOGIC;
	  --Ties							: IN STD_LOGIC;
	  HiPlayerBal					: IN STD_LOGIC_VECTOR(11 downto 0);
	  betReady 						: OUT STD_LOGIC;
	  face_chance					: IN STD_LOGIC_VECTOR(2 downto 0);
      bet                           : OUT STD_LOGIC_VECTOR(11 DOWNTO 0));
  END COMPONENT;


  signal Action     : STD_LOGIC_VECTOR (2*nPlayers-1 downto 0);
  signal TakeAction : STD_LOGIC;
  signal bet : STD_LOGIC_VECTOR(11 downto 0);
  signal ccard : std_logic_vector(3 downto 0);
  signal over_reset : std_logic;
  signal house_card_dealt : std_logic;
  signal shuffle: std_logic;
  SIGNAL NumA   : STD_LOGIC_VECTOR (4 downto 0);
  SIGNAL Num2   : STD_LOGIC_VECTOR (4 downto 0);
  SIGNAL Num3 : STD_LOGIC_VECTOR (4 downto 0);
  SIGNAL Num4  : STD_LOGIC_VECTOR (4 downto 0);
  SIGNAL Num5  : STD_LOGIC_VECTOR (4 downto 0);
  SIGNAL Num6   : STD_LOGIC_VECTOR (4 downto 0);
  SIGNAL Num7 : STD_LOGIC_VECTOR (4 downto 0);
  SIGNAL Num8 : STD_LOGIC_VECTOR (4 downto 0);
  SIGNAL Num9  : STD_LOGIC_VECTOR (4 downto 0);
  SIGNAL Num10   : STD_LOGIC_VECTOR (4 downto 0);
  SIGNAL NumJ  : STD_LOGIC_VECTOR (4 downto 0);
  SIGNAL NumQ : STD_LOGIC_VECTOR (4 downto 0);
  SIGNAL NumK  : STD_LOGIC_VECTOR (4 downto 0);


  SIGNAL card        : std_logic_vector(3 downto 0);
  SIGNAL dealerShow  : std_logic_vector(3 downto 0);
  SIGNAL issued      : std_logic;

  SIGNAL clock       : STD_LOGIC;
  SIGNAL enable      : STD_LOGIC;
  SIGNAL reset       : STD_LOGIC;
  SIGNAL bistEnable  : STD_LOGIC:='0';
  SIGNAL pHit        : STD_LOGIC_VECTOR(nPlayers-1 DOWNTO 0);
  SIGNAL pStand      : STD_LOGIC_VECTOR(nPlayers-1 DOWNTO 0);
  SIGNAL gameOver    : STD_LOGIC;
  SIGNAL balanceRdy  : STD_LOGIC;
  SIGNAL skBust      : STD_LOGIC;
  SIGNAL playerID    : STD_LOGIC_VECTOR(nPlayers-1 DOWNTO 0);
  SIGNAL globalReset : STD_LOGIC;
  SIGNAL cgIssue     : STD_LOGIC;
  SIGNAL cgNewHand   : STD_LOGIC;
  SIGNAL cgBist      : STD_LOGIC;
  SIGNAL handOver    : STD_LOGIC;

  SIGNAL numHands                      : STD_LOGIC_VECTOR(6 DOWNTO 0);
  SIGNAL balance                       : STD_LOGIC_VECTOR(11 DOWNTO 0);

  SIGNAL cgCARD : STD_LOGIC_VECTOR(3 DOWNTO 0);

  --SIGNAL Ties : STD_LOGIC;
  SIGNAL HiPlayerBal : STD_LOGIC_VECTOR(11 downto 0);
  SIGNAL betReady : STD_LOGIC;
  SIGNAL face_chance : STD_LOGIC_VECTOR(2 downto 0);
  SIGNAL dealttotal_out : std_logic_vector(7 downto 0);
begin  -- structure

  banker_1: banker
    generic map (
      n => nPlayers)
    port map (
      Clk        => clock,
      Reset      => globalReset,
      playerId   => playerId,
      Action     => Action,
      TakeAction => TakeAction,
      cgNewHand  => cgNewHand,
      bet(11 downto 0)        => bet,
      --bet(11 downto 10) => "00",
	  betReady => betReady,
      balance(11 downto 0)    => balance,
    -- balance(11 downto 10) => "00",
	  --Ties      => Ties,
	  HiPlayerBal => HiPlayerBal,
      GameOver   => gameOver,
      balancerdy => balanceRdy);

  Bjsk_1: Bjsk
    generic map (
      n => nPlayers)
    port map (
      Clk        => clock,
--    Reset      => globalReset,
      Reset      => cgNewHand,
      Card       => card,
      PlayerId   => PlayerId,
      cgIssue    => cgIssue,
      PlStand    => handOver,            -- What is this?
      GameOver   => globalReset,
      Action     => Action,
      TakeAction => TakeAction,
      skBust     => skBust);


  
  controller_1: controller
    GENERIC MAP (
      nPlayers => nPlayers)
    PORT MAP (
      clock       => clock,
      enable      => enable,
      reset       => reset,
      bistEnable  => bistEnable,
      pHit        => pHit,
      pStand      => pStand,
      issued      => issued,
      gameOver    => gameOver,
      balanceRdy  => balanceRdy,
      skBust      => skBust,
      playerID    => playerID,
      globalReset => globalReset,
      cgIssue     => cgIssue,
      cgNewHand   => cgNewHand,
      cgBist      => cgBist,
      handOver    => handOver,
	  shuffle     => shuffle
	  );

cardcounter_1: cardcounter
    PORT MAP (
      clock      => clock,
      reset      => reset,
     issued     => issued,
      card       => ccard,
      over_reset => over_reset,
      house_card_dealt  =>  house_card_dealt,
      stat_ace   => NumA,
      stat_two   => Num2,
      stat_three => Num3,
      stat_four  => Num4,
      stat_five  => Num5,
      stat_six   => Num6,
      stat_seven => Num7,
      stat_eight => Num8,
      stat_nine  => Num9,
      stat_ten   => Num10,
      stat_jack  => NumJ,
      stat_queen => NumQ,
      stat_king  => NumK,
	  dealttotal_out => dealttotal_out,
      face_chance => face_chance);

  
  card_generator_1: card_generator
    PORT MAP (
      in_clock      => clock,
      in_reset      => cgNewHand,
      in_issue      => cgissue,
      in_bist       => cgbist,
      main_reset => reset,
      shuffle => shuffle,
      in_num_twos   => Num2,
      in_num_threes => Num3,
      in_num_fours  => Num4,
      in_num_fives  => Num5,
      in_num_sixes  => Num6,
      in_num_sevens => Num7,
      in_num_eights => Num8,
      in_num_nines  => Num9,
      in_num_tens   => Num10,
      in_num_jacks  => NumJ,
      in_num_kings  => NumK,
      in_num_queens => NumQ,
      in_num_aces   => NumA,
      out_card      => card,
      out_house     => dealerShow,
      out_issued    => issued,
	  dealttotal_out=> dealttotal_out, 
      over_reset =>over_reset,
      ccard => ccard,
      house_card_dealt =>house_card_dealt);
      
  housePlayer_1: housePlayer
    PORT MAP (
      clock      => clock,
      reset      => reset,
      turn       => playerID(nPlayers-1),
      issued     => issued,
      card       => card,
      dealerShow => dealerShow,
      NumA       => NumA,
      Num2       => Num2,
      Num3       => Num3,
      Num4       => Num4,
      Num5       => Num5,
      Num6       => Num6,
      Num7       => Num7,
      Num8       => Num8,
      Num9       => Num9,
      Num10      => Num10,
      NumJ       => NumJ,
      NumQ       => NumQ,
      NumK       => NumK,
      numHands   => numHands,
      balance    => balance,
      balanceRdy => balanceRdy,
      hit        => pHit(nPlayers-1),
      stand      => pStand(nPlayers-1),
      bet        => bet);

  basicPlayer_1: basicPlayer
    PORT MAP (
      clock      => clock,
      reset      => reset,
      turn       => playerID(0),
      issued     => issued,
      card       => card,
      dealerShow => dealerShow,
      NumA       => NumA,
      Num2       => Num2,
      Num3       => Num3,
      Num4       => Num4,
      Num5       => Num5,
      Num6       => Num6,
      Num7       => Num7,
      Num8       => Num8,
      Num9       => Num9,
      Num10      => Num10,
      NumJ       => NumJ,
      NumQ       => NumQ,
      NumK       => NumK,
      numHands   => numHands,
      balance    => balance,
      balanceRdy => balanceRdy,
      hit        => pHit(0),
      stand      => pStand(0),
	  --Ties       => Ties,
	  HiPlayerBal=> HiPlayerBal,
	  betReady   => betReady,
	  face_chance=> face_chance,
      bet        => bet);


  basicPlayer_2: fileBasicPlayer
    PORT MAP (
      clock      => clock,
      reset      => reset,
      turn       => playerID(1),
      issued     => issued,
      card       => card,
      dealerShow => dealerShow,
      NumA       => NumA,
      Num2       => Num2,
      Num3       => Num3,
      Num4       => Num4,
      Num5       => Num5,
      Num6       => Num6,
      Num7       => Num7,
      Num8       => Num8,
      Num9       => Num9,
      Num10      => Num10,
      NumJ       => NumJ,
      NumQ       => NumQ,
      NumK       => NumK,
      numHands   => numHands,
      balance    => balance,
      balanceRdy => balanceRdy,
      hit        => pHit(1),
      stand      => pStand(1),
	  --Ties       => Ties,
	  HiPlayerBal=> HiPlayerBal,
	  betReady   => betReady,
	  face_chance=> face_chance,
      bet        => bet);
  

init_proc: PROCESS
BEGIN  -- PROCESS init_proc
  reset <= '1';

  WAIT FOR 80 ns;
  reset <= '0';


  -- ScoreKeeper should do this after the hands are played
  wait until gameOver = '1';
  report "gameOver state reached";
  wait;
--  reset <= '1';

END PROCESS init_proc;


--fake_bank : PROCESS (playerID)
--BEGIN
--  IF playerID(0) = '1' THEN
--    
--  END IF;
--  
--  IF handOver = '1' THEN
--    balanceRdy <= '1';
--  ELSE
--    balanceRdy <= '0';
--  END IF;
--
--END process;
  

clock_proc : PROCESS
begin
  --this process was generated based on formula: 0 0 ns, 1 20 ns -r 40 ns
  --wait for <time to next event>; -- <current time>
  clock <= '0';
  wait for 20 ns; --0 fs
  
  clock <= '1';
  wait for 20 ns; --20 ns
end process clock_proc;
--end process;
end structure;
